if instead.tiny then
  -- Stub the theme if instead.tiny is present
  function set_theme(t) end;
  return;
end;

function set_theme(t)  
  -- resetting defaults
  fmt.para = true;
  std.scene_delim = '^^';
  theme.set('snd.click', 'sound/click.wav');
  
	D {'settings_open'};

  local tname, tcolor_text, tcolor_link, tcolor_alink;

  local width, height = diretheme.get_screen();
	
  -- 640x480 is the base size
	local scale = get_graphic_scale();
	local max_scale_x = math.floor(width / 640);
	local max_scale_y = math.floor(height / 480);
	-- Fail-safe for too small screen
  if max_scale_x < 1 then max_scale_x = 1; end;
  if max_scale_y < 1 then max_scale_y = 1; end;
	  
  local max_scale = math.min(max_scale_x, max_scale_y);
  
  if scale > max_scale then
    scale = max_scale;
    prefs.graphic_scale = false;
  end;
  
  local cursor_scale = get_cursor_scale();
  if cursor_scale > 2 * max_scale then
    cursor_scale = 2 * max_scale;
  end;
  
	local font_scale = get_font_scale() / 100;
	if font_scale > 2 * scale then
		font_scale = scale;
		prefs.font_scale = font_scale * 100
	end;
	
  local inv_scale = get_inventory_font_scale() / 100;
	
	local text_align = 'justify';
	if prefs.text_alignment == 2 then
		text_align = 'left';
	end;
	
	local hpadding, vpadding = 0, 0;
	if prefs.text_padding == 2 then
		hpadding, vpadding = 24, 12;
	end;
	
	local interval = '1.0';
	if prefs.text_interval == 2 then
		interval = '1.5';
	end;
	
	local canvas = diretheme.set_canvas({rmax = 2, rmin = 1});
	diretheme.set_scale(scale);

  local scaled_width, scaled_height = diretheme.get_scaled_screen();

	current_theme = t;
  local tvariant = 0;
  if t:match("_bw$") then
    t = t:gsub("_bw$", "");
    tvariant = 1;
  end;
  
  -- Most text area settings are per-theme
  local text_area = {};
  
  -- Hide inventory by default
  local inventory = {
    rect = {
      lx = width + 1,
      y = 0,
      wmax = 1,
      wmin = 1,
      ty = 2,
      by = 2,
      ax = 1
    }
  };
  
  -- And image
  local image_area = {
    rect = {
      lx = width + 1,
      ax = 1,
      wmax = 584,
      hmax = 258,
      wmin = 584,
      hmin = 258
    };
    bg = {};
    mode = 'float';
  };
  
  -- Preset colors
  local theme_directory = nil;
  local text_color, link_color, alink_color;
  local inv_color, inv_link, inv_alink;
  if t == 'terminal' or t == 'black' then
    diretheme.set_canvas({
      lx = 0, ty = 0, rx = 0, by = 0
    });
    text_color = '#D0FFFF';
    link_color = '#ABDECE';
    alink_color = '#9BCEBE';  
    theme_directory = 'images/tower_theme';
    
    text_area.text = {
      font = 'fonts/vga.ttf',
      color = text_color,
      link = link_color,
      alink = alink_color,
      size = 16
    };
  else
    canvas = diretheme.set_canvas({
      lx = 0, ty = 0, rx = 0, by = 0, rmin = 1.2, rmax = 1.8
    });
    
    if t == 'menu' or t == 'main' or t == 'ending_outside' or t == 'credits_outside' then
      inv_color = '#403426';
      inv_link = '#403426';
      inv_alink = '#61503A';
      
      if tvariant == 0 then
        text_color = '#000504';
        link_color = '#80382A';
        alink_color = '#80605D';
        theme_directory = 'images/main_theme';
      else
        text_color = '#040404';
        link_color = '#434343';
        alink_color = '#585858';
        theme_directory = 'images/bwm_theme';
      end;
      
      text_area.uppos = {ax=1, x = 12, y = 0, ay=-1};
      text_area.downpos = {ax=1, x = 12, y = 0, ay=1};
      text_area.bg = {
        image = theme_directory .. '/text_bg.png',
        options = {
          mode = 6,
          drawmethod = 2,
          slice = {
            x = 9,
            y = 6,
            w = 96,
            h = 96
          },
          scale = true
        }
      };
    elseif t == 'tower' or t == 'ending_inside' or t == 'credits_inside' then
      inv_color = '#6b9e8e';
      inv_link = '#9bcebe';
      inv_alink = '#Bbeede';
      
      if tvariant == 0 then
        text_color = '#93DDCD';
        link_color = '#E0F0EA';
        alink_color = '#FFFFFF';
        theme_directory = 'images/tower_theme';
      else
        text_color = '#ADADAD';
        link_color = '#CDCDCD';
        alink_color = '#BDBDBD';
        theme_directory = 'images/bwt_theme';
      end;
      
      text_area.uppos = {ax=1, x = 16, y = 0, ay=-1};
      text_area.downpos = {ax=1, x = 16, y = 0, ay=1};
      text_area.bg = {
        image = theme_directory .. '/text_bg.png',
        options = {
          mode = 6,
          drawmethod = 2,
          slice = {
            x = 6,
            y = 6,
            w = 128,
            h = 128
          },
          scale = true
        }
      };
    end;
    
    if t == 'ending_inside' or t == 'ending_outside' then
      canvas = diretheme.set_canvas({
        lx = 0, ty = 0, rx = 0, by = 0
      });
    end;
  
    text_area.align = text_align;
    text_area.height = interval;
    text_area.scaleui = true;
    text_area.up = theme_directory .. '/arrow_up.png';
    text_area.down = theme_directory .. '/arrow_down.png';
    text_area.text = {
      font = 'fonts/{sans,sans-b,sans-i,sans-bi}.ttf',
      color = text_color,
      link = link_color,
      alink = alink_color,
      size = math.floor(16 * font_scale),
      noscale = true
    };
    
    inventory.up = theme_directory .. '/arrow_up.png';
    inventory.down = theme_directory .. '/arrow_down.png';
    inventory.uppos = {ax=1, x = 8, y = 0, ay=-1};
    inventory.downpos = {ax=1, x = 8, y = 0, ay=1};
    inventory.text = {
      color = inv_color,
      link = inv_link,
      alink = inv_alink,
      size = math.floor(16 * inv_scale),
      noscale = true
    };
    inventory.height = interval;
  end;
  
  if t == 'main' or t == 'tower' then
    std.scene_delim = '^' .. fmt.c(fmt.img(theme_directory .. '/line_separator.png')) .. '^'
  end;
  
 -- And menu, but need theme_directory to have proper button image
  local menu = {
    button = {
      image = theme_directory ..'/menu.png';
      options = {
        mode = 1,
        scale = true
      }
    };
    buttonrect = {
      x = width + 1,
      y = 0,
      w = 50,
      h = 16
    };
    text = {
      size = math.floor(15 * font_scale),
      noscale = true
    },
    height = interval,
    scaleui = true;
  };
  
  local cursor, active_cursor = 'images/cursor.png', 'images/cursor_active.png';
  if tvariant == 1 then
    cursor, active_cursor = 'images/cursor_bw.png', 'images/cursor_active_bw.png'
  end;
  
  diretheme.set_cursor({
		x = 2,
		y = 2,
		normal = cursor,
		active = active_cursor,
    scale = true;
		scalefactor = cursor_scale;
	});
  
	if t == 'menu' then
		diretheme.set_background('#89C6AE', 255, 'images/main_theme/background.png', {mode=2, scale=true});
    local padding_delta = -10;
    if height < 520 * scale then
      padding_delta = -95;
    end;
    
    text_area.rect = {
      ty = padding_delta + vpadding,
      wmax = 538 - hpadding * 2,
      hmax = 322 - vpadding * 2,
      ay = 0
    };
    text_area.text = text_area.text;
    text_area.uppos = {ax=1, x = 15, y = 65, ay=-1};
    text_area.downpos = {ax=1, x = 15, y = -20, ay=1};
    text_area.contentrect = {wf=prefs.text_width / 100};
    text_area.bg = '';
	elseif t == 'main' then
		diretheme.set_background('#89C6AE', 255, 'images/main_theme/background.png', {mode=2, scale = true});
		
		local image_bg = 'images/main_theme/picture_bg.png';
		if canvas.w < 800 * scale then
			image_bg = 'images/main_theme/picture_bg_small.png';
		end;
		
    image_area.leftof = 'inv';
    image_area.topof = 'text';
    image_area.rect = {
      ax = 0,
      wmax = 584,
      hmax = 265,
      wmin = 584,
      hmin = 265
    };
    image_area.contentrect = {
      ax = 0,
      ay = -1,
      ty = 11,
      wmin = 320,
      hmin = 240,
      wmax = 320,
      hmax = 240
    };
    image_area.bg = {
      image = image_bg,
      options = {
        mode = 1,
        scale = true
      }
    };
		
    text_area.leftof = 'inv';
    text_area.rect = {
      lx = 7,
      ty = 266,
      rx = 8,
      by = 2,
      ax = 1,
      ay = 1
    };
    text_area.contentrect = {
      lx = 21 + hpadding,
      rx = 21 + hpadding,
      ty = 14 + vpadding,
      by = 14 + vpadding,
      wf=prefs.text_width / 100
    };
    
		inventory.rect = {
      rx = 2,
      y = 2,
      wmax = 208,
      wmin = 208,
      ty = 2,
      by = 2,
      ax = 1
    };
    inventory.contentrect = {
      lx = 25,
      rx = 25,
      ty = 40,
      by = 40
    };
    inventory.bg = {
      image = function(tpx, rect)
        local top = pixels.new('images/main_theme/inv_top.png'):scale(scale, scale, false);
        local mid = pixels.new('images/main_theme/inv_mid.png'):scale(scale, scale, false);
        local bottom = pixels.new('images/main_theme/inv_bottom.png'):scale(scale, scale, false);
        
        dire_draw_tile(mid, tpx, { x=rect.x + 11 * scale, y=rect.y + 10 * scale, w=rect.w - 22 * scale, h = rect.h - 20 * scale}, {});
        dire_draw_center(top, tpx, {x=rect.x, y=rect.y, w=208 * scale, h=60 * scale});
        dire_draw_center(bottom, tpx, {x=rect.x, y=rect.y + rect.h - 60 * scale, w=208 * scale, h=60 * scale});
      end,
      options = {scale = true}
    };

    menu.buttonrect = {
      wmin = 50,
      wmax = 50,
      hmin = 16,
      hmax = 16,
      by = 12,
      ay = 1,
      rx = 81,
      ax = 1
    };

		D {
			'settings_open', 
			'img', 
			get_sprite('images/main_theme/settings.png'),
			x=canvas.x + canvas.w - 33 * scale, 
			y=canvas.y + canvas.h - 30 * scale, 
			click=true,
			z=-1,
      background=true
		};
	elseif t == 'tower' then
		diretheme.set_background('#181E37', 255, 'images/tower_theme/background.png', {mode=2, scale=true});
		
		local image_bg = 'images/tower_theme/picture_bg.png';
		if canvas.w < 800 * scale then
			image_bg = 'images/tower_theme/picture_bg_small.png';
		end;
		
    image_area.leftof='inv';
    image_area.topof='text';
    image_area.rect = {
      ax = 0,
      maxw = 584,
      maxh = 258,
      minw = 584,
      minh = 258
    };
    image_area.contentrect = {
      ax = 0,
      ay = -1,
      ty = 12,
      wmin = 320,
      hmin = 240,
      wmax = 320,
      hmax = 240
    };
    image_area.bg = {
      image = image_bg,
      options = {
        mode = 1,
        scale = true
      }
    };
		
    text_area.leftof = 'inv';
    text_area.rect = {
      lx = 7,
      ty = 266,
      rx = 8,
      by = 2,
      ax = 1,
      ay = 1
    };
    text_area.contentrect = {
      lx = 21 + hpadding,
      rx = 21 + hpadding,
      ty = 14 + vpadding,
      by = 14 + vpadding,
      wf = prefs.text_width / 100
    };
  
    inventory.rect = {
      rx = 2,
      y = 2,
      wmax = 208,
      wmin = 208,
      ty = 2,
      by = 2,
      ax = 1
    };
    inventory.contentrect = {
      lx = 12,
      rx = 16,
      ty = 10,
      by = 40
    };
    inventory.bg = {
      image = 'images/tower_theme/inv_bg.png',
      options = {
        mode = 6,
        drawmethod = 2,
        slice = {
          x = 7,
          y = 11,
          w = 194,
          h = 38
        }, 
        scale = true
      }
    };
    
    menu.buttonrect = {
      wmin = 50,
      wmax = 50,
      hmin = 16,
      hmax = 16,
      by = 20,
      ay = 1,
      rx = 81,
      ax = 1
    };
		
		D {
			'settings_open', 
			'img', 
			get_sprite('images/tower_theme/settings.png'),
			x=canvas.x + canvas.w - 35 * scale, 
			y=canvas.y + canvas.h - 37 * scale,
			click=true,
			z=-1,
      background=true
		};
	elseif t == 'terminal' then
		theme.set('snd.click', '');
		
    local upshift = 0;
    if height / scale < 540 then
      upshift = 19;
    end;
    
    local px = pixels.new(scaled_width, scaled_height);
		local tiles_src = pixels.new('images/terminal/table_tiles.png');
    local backdrop_src = pixels.new('images/terminal/backdrop.png');
		
		local tabletop = scaled_height - 422 - upshift
		if tabletop < 335 - upshift then
			tabletop = 335 - upshift;
		end;
		
		dire_draw_tile(tiles_src, px, {
			w = scaled_width,
			h = 422,
			x = 0,
			y = tabletop
		}, {xoffset = -1, yoffxet = -1});

    dire_draw_tile(backdrop_src, px, {
			w = scaled_width,
			h = tabletop,
			x = 0,
			y = 0
		}, {xoffset = -1, yoffset = -2});
		
		diretheme.set_background('#181E37', 255, px, {mode=5});
		
		local ty
		if here().ticks ~= nil and here().ticks < 128 and here().graphical then
			ty = 72
		else
			ty = 2
		end;		

    text_area.rect = {
      ty = 0 - upshift,
      by = 189,
      wmin = 808,
      hmin = 568,
      wmax = 808,
      hmax = 568,
      ax = 0,
      ay = 1
    };
    text_area.contentrect = {
      wmax = 569,
      hmax = 390,
      ty = ty,
      ax = 0,
      ay = -2,
      dx = -16
    };
    text_area.bg = {
      image = 'images/terminal/bg_text.png',
      options = {
        mode = 1,
        scale = true
      }
    };
	elseif t == 'ending_outside' then
		diretheme.set_background('#89C6AE', 255, theme_directory .. '/background.png', {mode=2, scale=true});
		
    text_area.rect = {
      lx = 24,
      rx = 24,
      wmax = 600,
      hmax = 160,
      by = 24,
      ay = 1
    };
    text_area.contentrect = {
      lx = 18 + hpadding,
      rx = 18 + hpadding,
      ty = 14 + vpadding,
      by = 14 + vpadding,
      wf = prefs.text_width / 100
    };
	elseif t == 'ending_inside' then
		diretheme.set_background('#181E37', 255, theme_directory .. '/background.png', {mode=2});
		
    text_area.rect = {
      lx = 24,
      rx = 24,
      wmax = 600,
      hmax = 160,
      by = 24,
      ay = 1
    };
    text_area.contentrect = {
      lx = 18 + hpadding,
      rx = 18 + hpadding,
      ty = 14 + vpadding,
      by = 14 + vpadding,
      wf = prefs.text_width / 100
    };
  elseif t == 'credits_outside' or t == 'credits_inside' then
    -- text scaled on top of picture
    text_area.rect = {
      wmax = 960,
      hmax = 600
    };
    text_area.bg = {};
    text_area.align = 'center';
    text_area.content_rect = {
      wf = prefs.text_width / 100
    };
	elseif t == 'black' then    
		fmt.para = false;
		diretheme.set_background('#000000', 255);
		
		text_area.rect = {
      wmax = 600,
      hmax = 400,
      ax = 0,
      ay = 0
    };
    text_area.align = 'center';
	end;
  
  diretheme.set_image_area(image_area, true);
  diretheme.set_text_area(text_area, true);
  diretheme.set_inventory_area(inventory, true);
  
  diretheme.set_menu(menu, true);
end;