local keysfilter = false;

function set_global_keysfilter(func)
  keysfilter = func
end;

function keys:filter(press, key)
	local rval;
	if here().keysfilter then
		return here():keysfilter(press, key);
	end;
	if keysfilter then
		return keysfilter(press, key);
	end;
end

--[[
	Walk from links
]]
obj {
	nam = '@walk';
	act = function(s, w)
		walk(w)
	end;
}

obj {
	nam = '@walkin';
	act = function(s, w)
		walkin(w)
	end;
}

function random_phrase(phrases)
  return phrases[rnd(#phrases)];
end;

--[[
	Raw text display
]]
obj {
	nam = '$raw';
	act = function(s)
		local r = std.call(here(), 'rawdsc');
		r = string.gsub(r, '\\?[\\^]', { ['^'] = '\n', ['\\^'] = '^'} );

		local spaces = false;
		local state = '';

		for token in string.gmatch(r, '.') do
			if spaces then
				if token ~= ' ' then
					spaces = false;
					pr(fmt.nb(state));
					state = token;
				else
					state = state .. token;
				end;
			else
				if token == ' ' then
					spaces = true;
					pr(state);
					state = token;
				else
					state = state .. token;
				end;
			end;
		end;

		if spaces then
			pr(fmt.nb(state));
		else
			pr(state);
		end;
	end;
}

--[[
	Fading control
]]
local oldfading = instead.fading

instead.fading = function()
	if here().nofading then
		return false;
	else
		return oldfading()
	end;
end

local __ds = false;

function has_object (tab, val)
	for index, value in ipairs(tab) do
		if value.nam == val then
			return true
		end
	end

	return false
end;

function Set (list)
  local set = {}
  for _, l in ipairs(list) do set[l] = true end
  return set
end

local old_dlg = dlg;

dlg = function(s)
	s.noinv = true;
	s.nofollow = true;
	return old_dlg(s);
end;

function table.copy(t)
  local t2 = {};
  for k,v in pairs(t) do
	if type(v) == "table" then
		t2[k] = table.copy(v);
	else
		t2[k] = v;
	end
  end
  return t2;
end

pathout = std.class({
	__path_type = true;
	new = function(_, t)
		if type(t) ~= 'table' then
			std.err("Wrong path argument. Use {} as path argument.", 2)
		end
		local n, s = t[1], t[2]

		if not s then
			s = n
			n = nil
		end

		local new = {
			before = s;
		}

		for k, v in std.pairs(t) do
			if type(k) == 'string' then
				new[k] = v
			end
		end

		new.nam = n or new.nam

		return std.room(new)
	end;
	disp = function(s)
		if s.after ~= nil and visited(w) then
			return std.call(s, 'after')
		end
		return std.call(s, 'before')
	end;
	onwalk = function(s, _)
		local r = here():from();
		walk(r, true, false, false)
		return false
	end;
}, std.room)

function robj(t)
	if not t.way then
		t.way = {
			pathout {'Назад'};
		};
	else
		table.insert(t.way, pathout {'Назад'});
	end;
	return room(t);
end;

function split(text, delim)
    -- taken from https://stackoverflow.com/questions/1426954/split-string-in-lua
    -- returns an array of fields based on text and delimiter (one character only)
    local result = {}
    local magic = "().%+-*?[]^$"

    if delim == nil then
        delim = "%s"
    elseif string.find(delim, magic, 1, true) then
        -- escape magic
        delim = "%"..delim
    end

    local pattern = "[^"..delim.."]+"
    for w in string.gmatch(text, pattern) do
        table.insert(result, w)
    end
    if #result == 0 then
      result[1] = '';
    end;
    return result
end;

function scale_frame_data(fd)
  local scale = diretheme.get_scale();
  for _, frame in ipairs(fd) do
    if frame['x'] then
      frame['x'] = frame['x'] * scale;
    end;
    if frame['y'] then
      frame['y'] = frame['y'] * scale;
    end;
    if frame['w'] then
      frame['w'] = frame['w'] * scale;
    end;
    if frame['h'] then
      frame['h'] = frame['h'] * scale;
    end;
    if frame['dx'] then
      frame['dx'] = frame['dx'] * scale;
    end;
    if frame['dy'] then
      frame['dy'] = frame['dy'] * scale;
    end;
  end;
  
  return fd;
end;

function cat_list(list, sep)
	local rval;

	if sep == nil then
		sep = '';
	end;

	for i, v in ipairs(list) do
		if rval then
			rval = rval .. sep .. v;
		else
			rval = v;
		end;
	end;

	return rval;
end;

